#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>

#define AEC_FUNCTIONS
#include "AkelEdit.h"
#include "AkelDLL.h"


//Functions prototypes
LRESULT CALLBACK NewEditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL SmartHomeA(HWND hWnd);
BOOL SmartHomeW(HWND hWnd);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
HWND hMainWnd;
BOOL bOldWindows;
BOOL bAkelEdit;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
WNDPROCDATA *NewEditProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="SmartHome";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

LRESULT CALLBACK NewEditProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_KEYDOWN || uMsg == WM_SYSKEYDOWN)
  {
    if (wParam == VK_HOME)
    {
      if (GetKeyState(VK_CONTROL) >= 0)
      {
        if (bAkelEdit)
        {
          //AkelEdit
          AESELECTION aes;
          AECHARINDEX ciCaret;
          int i;

          SendMessage(hWnd, AEM_GETSEL, (WPARAM)&ciCaret, (LPARAM)&aes);
          for (i=0; ciCaret.lpLine->wpLine[i] == ' ' || ciCaret.lpLine->wpLine[i] == '\t'; ++i);

          if (i)
          {
            if (ciCaret.nCharInLine > i || ciCaret.nCharInLine == 0)
            {
              if (!AEC_IndexCompare(&ciCaret, &aes.crSel.ciMin))
              {
                ciCaret.nCharInLine=i;
                aes.crSel.ciMin=ciCaret;
              }
              else
              {
                ciCaret.nCharInLine=i;
                aes.crSel.ciMax=ciCaret;
              }

              if (GetKeyState(VK_SHIFT) >= 0)
              {
                aes.crSel.ciMin=ciCaret;
                aes.crSel.ciMax=ciCaret;
              }
              SendMessage(hWnd, AEM_SETSEL, (WPARAM)&ciCaret, (LPARAM)&aes);

              return TRUE;
            }
          }
        }
        else
        {
          //RichEdit
          if (bOldWindows)
          {
            if (SmartHomeA(hWnd))
              return TRUE;
          }
          else
          {
            if (SmartHomeW(hWnd))
              return TRUE;
          }
        }
      }
    }
  }

  //Call next procedure
  return NewEditProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

BOOL SmartHomeA(HWND hWnd)
{
  CHARRANGE crCurrent;
  CHARRANGE crNew;
  TEXTRANGEA txtrng;
  char *pText;
  int nMinLine;
  int nMinLineIndex;
  int nMinLineLength;
  int i;
  BOOL bResult=FALSE;

  SendMessage(hWnd, EM_EXGETSEL, 0, (LPARAM)&crCurrent);
  nMinLine=SendMessage(hWnd, EM_EXLINEFROMCHAR, 0, crCurrent.cpMin);
  nMinLineIndex=SendMessage(hWnd, EM_LINEINDEX, nMinLine, 0);
  nMinLineLength=SendMessage(hWnd, EM_LINELENGTH, nMinLineIndex, 0);

  if (pText=(char *)GlobalAlloc(GPTR, nMinLineLength + 1))
  {
    txtrng.chrg.cpMin=nMinLineIndex;
    txtrng.chrg.cpMax=nMinLineIndex + nMinLineLength;
    txtrng.lpstrText=pText;
    SendMessageA(hWnd, EM_GETTEXTRANGE, 0, (LPARAM)&txtrng);

    for (i=0; pText[i] == ' ' || pText[i] == '\t'; ++i);

    if (i)
    {
      if ((crCurrent.cpMin - nMinLineIndex) > i || (crCurrent.cpMin - nMinLineIndex) == 0)
        crNew.cpMin=nMinLineIndex + i;
      else
        crNew.cpMin=nMinLineIndex;

      if (GetKeyState(VK_SHIFT) < 0)
        crNew.cpMax=crCurrent.cpMax;
      else
        crNew.cpMax=crNew.cpMin;
      SendMessage(hWnd, EM_EXSETSEL, 0, (LPARAM)&crNew);
      bResult=TRUE;
    }
    GlobalFree((HGLOBAL)pText);
  }
  return bResult;
}

BOOL SmartHomeW(HWND hWnd)
{
  CHARRANGE crCurrent;
  CHARRANGE crNew;
  TEXTRANGEW txtrng;
  wchar_t *wpText;
  int nMinLine;
  int nMinLineIndex;
  int nMinLineLength;
  int i;
  BOOL bResult=FALSE;

  SendMessage(hWnd, EM_EXGETSEL, 0, (LPARAM)&crCurrent);
  nMinLine=SendMessage(hWnd, EM_EXLINEFROMCHAR, 0, crCurrent.cpMin);
  nMinLineIndex=SendMessage(hWnd, EM_LINEINDEX, nMinLine, 0);
  nMinLineLength=SendMessage(hWnd, EM_LINELENGTH, nMinLineIndex, 0);

  if (wpText=(wchar_t *)GlobalAlloc(GPTR, nMinLineLength * sizeof(wchar_t) + 2))
  {
    txtrng.chrg.cpMin=nMinLineIndex;
    txtrng.chrg.cpMax=nMinLineIndex + nMinLineLength;
    txtrng.lpstrText=wpText;
    SendMessageW(hWnd, EM_GETTEXTRANGE, 0, (LPARAM)&txtrng);

    for (i=0; wpText[i] == ' ' || wpText[i] == '\t'; ++i);

    if (i)
    {
      if ((crCurrent.cpMin - nMinLineIndex) > i || (crCurrent.cpMin - nMinLineIndex) == 0)
        crNew.cpMin=nMinLineIndex + i;
      else
        crNew.cpMin=nMinLineIndex;

      if (GetKeyState(VK_SHIFT) < 0)
        crNew.cpMax=crCurrent.cpMax;
      else
        crNew.cpMax=crNew.cpMin;
      SendMessage(hWnd, EM_EXSETSEL, 0, (LPARAM)&crNew);
      bResult=TRUE;
    }
    GlobalFree((HGLOBAL)wpText);
  }
  return bResult;
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hMainWnd=pd->hMainWnd;
  bOldWindows=pd->bOldWindows;
  bAkelEdit=pd->bAkelEdit;
}

void InitMain()
{
  bInitMain=TRUE;

  //SubClass edit procedure
  NewEditProcData=NULL;
  SendMessage(hMainWnd, AKD_SETEDITPROC, (WPARAM)NewEditProc, (LPARAM)&NewEditProcData);
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewEditProcData)
  {
    SendMessage(hMainWnd, AKD_SETEDITPROC, (WPARAM)NULL, (LPARAM)&NewEditProcData);
    NewEditProcData=NULL;
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
